# Migration from 4.x to 5.x

## Simplified DSL

### Elimination of Doneables

To reduce the number of classes generated by the client and the overall size of packages,
[Doneable][Doneable] interface, implementations, and implementation generator tasks have been removed.

Provided `Doneable` implementations (especially for CustomResources) are no longer needed and can be safely removed.

### Createable & CreateOrReplaceable

As a result of the `Doneable` interface removal, 
`createNew()` and `createOrReplaceWithNew()` methods are no longer available.
Usages of these methods should be refactored to use the `create(Resource... item)` or 
`createOrReplace(Resource... item)` methods instead.
```java
// Deprecated
client.configMaps().inNamespace("default").createNew()
  .withNewMetadata().withName("config-map-name").endMetadata()
  .addToData("key", "value")
  .done();
// Alternative
client.configMaps().inNamespace("default").create(
  new ConfigMapBuilder()
    .withNewMetadata().withName("config-map-name").endMetadata()
    .addToData("key", "value")
    .done();
);

// Deprecated
client.configMaps().inNamespace("default").createOrReplaceWithNew()
  .withNewMetadata().withName("config-map-name").endMetadata()
  .addToData("key", "value")
  .done();
// Alternative
client.configMaps().inNamespace("default").createOrReplaceWithNew(
  new ConfigMapBuilder()
    .withNewMetadata().withName("config-map-name").endMetadata()
    .addToData("key", "value")
    .done();
);
```

### Editable

As a result of the `Doneable` interface removal, `edit()` method is no longer available.
The lambda "friendly" [`T edit(UnaryOperator<T> function)`][edit-with-lambda] or the visitable approach
[`T edit(Class<V> visitorType, Visitor<V> visitor)`][edit-with-visitor] methods should be used instead.

```java
// Deprecated
client.configMaps().inNamespace("default").withName("config-map-name").edit()
  .addToData("key-2", "value-2")
  done();
// Alternative
client.configMaps().inNamespace("default").withName("config-map-name").edit(
  c -> new ConfigMapBuilder(c).addToData("key-2", "value-2").build()
);
// Alternative with visitable approach
client.configMaps().inNamespace("default").withName("config-map-name").edit(
  ObjectMetaBuilder.class,
  omb -> omb.addToAnnotations("key", "value")
);
```

## Refactored Watch

### Watcher interface

`Watcher#onClose(KubernetesClientException cause)` method has been replaced by `Watcher#onClose(WatcherException cause)`
and a new default `Watcher#onClose()` method has been added.

The argument-less onClose method is **always** invoked upon Watcher completion, regardless if there is an exception
or not. The `onClose` with exception argument is only invoked in case an exception occurs during the Watcher life-cycle
(once the watch connection is open and established).

For migration purposes, any onClose implementations resembling the following:
```java
void onClose(KubernetesClientException cause) {
  if (cause != null) {
    // Do in case of exception
  }
  // Clean-up operations
}
```
Should be refactored to:
```java
void onClose(WatcherException cause) {
  // Do in case of exception
}
void onClose() {
  // Clean-up operations
}
```

## Improved Custom Resource User Experience

This release should be a drop-in replacement for v4 in terms of breaking changes. However, the new release includes many
improvements that simplifies the usage of Custom Resources and Custom Resource Definitions in the client.

### Custom Resource POJO annotations

#### `@Version`
Allows to specify which version of the API the annotated class is defined under.

Together with `@Group`, this allows to determine the `apiVersion` field associated with the annotated resource.

See https://kubernetes.io/docs/reference/using-api/#api-versioning for more details.

#### `@Group`
Allows to specify which API group the annotated class is defined under.

Together with `@Version`, this allows to determine the `apiVersion` field associated with the annotated resource.

See https://kubernetes.io/docs/reference/using-api/#api-groups for more details.

#### `@Kind`
Allows to specify which `Kind` value should be used to refer to instance of the annotated class.

If not provided, a default value is computed based on the annotated class name.

See [HasMetadata#getKind][has-metadata-get-kind] for more details.

#### `@Singular`
Allows to specify the singular name of the resource.

#### `@Plural`
Allows to specify the plural name of the resource to serve.

The custom resources are served under `/apis/<group>/<version>/.../<plural>`

### Custom Resource Type Parameters

The [`CustomResource`][custom-resource] abstract class provides a default implementation for the convention fields
`spec` and `status`. The class is now parameterized and the Types for these fields can be specified through the new
Type parameters in the class.

### Helper & Utility methods

#### `CustomResourceDefinitionContext` - `v1CRDFromCustomResourceType` and `v1beta1CRDFromCustomResourceType`

New `v1CRDFromCustomResourceType` and `v1beta1CRDFromCustomResourceType` methods have been 
introduced on `CustomResourceDefinitionContext` to initialize a `CustomResourceDefinitionBuilder`
with the information provided by a specific `CustomResource` implementation.

These methods simplify the creation of `CustomResourceDefinition`s by inferring the boiler-plate spec values from
the annotations and other information already provided in the `CustomResource` implementation.

## kube-validation-schema.json files renamed to validation-schema.json

`kube-validation-schema.json` files are no longer included in our model packaged. The equivalent file
is now named `validation-schema.json`.

The [schema][json-schema] URL has been updated to the latest compatible [version][json-schema-draft-05]
(`http://json-schema.org/draft-05/schema#`)

[Doneable]: https://github.com/fabric8io/kubernetes-client/blob/d9c84fbc55b057526871764c6576a10219f98bdd/kubernetes-model-generator/kubernetes-model-core/src/main/java/io/fabric8/kubernetes/api/model/Doneable.java
[custom-resource]: https://github.com/fabric8io/kubernetes-client/blob/6a717e44e0600358068d87cc60628863a117f7f8/kubernetes-client/src/main/java/io/fabric8/kubernetes/client/CustomResource.java
[edit-with-lambda]: https://github.com/fabric8io/kubernetes-client/blob/6a717e44e0600358068d87cc60628863a117f7f8/kubernetes-client/src/main/java/io/fabric8/kubernetes/client/dsl/Editable.java#L25
[edit-with-visitor]: https://github.com/fabric8io/kubernetes-client/blob/6a717e44e0600358068d87cc60628863a117f7f8/kubernetes-client/src/main/java/io/fabric8/kubernetes/client/dsl/Editable.java#L29
[has-metadata-get-kind]: https://github.com/fabric8io/kubernetes-client/blob/6a717e44e0600358068d87cc60628863a117f7f8/kubernetes-model-generator/kubernetes-model-core/src/main/java/io/fabric8/kubernetes/api/model/HasMetadata.java#L54
[json-schema]: https://json-schema.org/understanding-json-schema/reference/schema.html
[json-schema-draft-05]: https://json-schema.org/specification-links.html#draft-5
